<?php

namespace NoccyLabs\Mattermost\DialogBuilder;

use JsonSerializable;
use NoccyLabs\Mattermost\DialogBuilder\Element\AbstractElement;
use ReflectionClass;

class DialogBuilder implements JsonSerializable
{
    // max 24 chars
    private ?string $title = null;

    private ?string $introductionText = null;

    private ?string $url = null;

    private ?string $triggerId = null;

    private ?string $callbackId = null;

    // max 5
    private array $elements = [];

    private ?string $iconUrl = null;

    private ?string $submitLabel = null;

    private ?bool $notifyOnCancel = null;
    
    private ?string $state = null;

    public static function create(...$args): DialogBuilder
    {
        return new DialogBuilder(...$args);
    }

    public function __construct(...$args)
    {
        $rc = new ReflectionClass($this);
        foreach ($args as $k=>$v) {
            $rp = $rc->getProperty($k);
            $rp->setValue($this,$v);
    
        }
    }

    public function setTriggerId(string $id): self
    {
        $this->triggerId = $id;
        return $this;
    }

    public function setCallbackId(string $id): self
    {
        $this->callbackId = $id;
        return $this;
    }

    public function setTitle(string $title): self
    {
        $this->title = $title;
        return $this;
    }

    public function setIntroductionText(string $text): self
    {
        $this->introductionText = $text;
        return $this;
    }

    public function setUrl(string $url): self
    {
        $this->url = $url;
        return $this;
    }

    public function addElement(string|AbstractElement $element, ...$args): self
    {
        if (!($element instanceof AbstractElement)) {
            $element = new $element(...$args);
        }
        $this->elements[] = $element;
        return $this;
    }

    public function jsonSerialize()
    {
        return [
            'url' => $this->url,
            'trigger_id' => $this->triggerId,
            'dialog' => $this->condense([
                'callback_id',
                'title',
                'introduction_text',
                'elements',
                'submit_label',
                'notify_on_cancel',
                'state'
            ])
        ];
    }

    private function condense(array $props)
    {
        $ret = [];
        $rc = new ReflectionClass($this);
        foreach ($props as $prop) {
            $cprop = strtr(ucwords(strtr($prop, "_", " ")), [' '=>'']);
            $cprop[0] = strtolower($cprop[0]);
            $rp = $rc->getProperty($cprop);
            $val = $rp->getValue($this);
            if ($val !== null) {
                $ret[$prop] = $val;
            }
        }
        return $ret;
    }
}
