<?php

namespace NoccyLabs\Mattermost\DialogBuilder\Element;

use JsonSerializable;
use ReflectionClass;

abstract class AbstractElement implements JsonSerializable
{

    protected ?string $displayName = null;

    protected ?string $name = null;

    protected ?bool $optional = null;

    protected ?string $helpText = null;

    protected ?string $default = null;

    public function __construct(...$args)
    {
        $rc = new ReflectionClass($this);
        foreach ($args as $k=>$v) {
            $rp = $rc->getProperty($k);
            $rp->setValue($this,$v);
        }
    }

    protected function makeElement(string $type): array
    {
        $el = [ 'type' => $type ];
        if ($this->displayName !== null) $el['display_name'] = (string)$this->displayName;
        if ($this->name !== null) $el['name'] = (string)$this->name;
        if ($this->optional !== null) $el['optional'] = (bool)$this->optional;
        if ($this->default !== null) $el['default'] = (string)$this->default;
        if ($this->helpText !== null) $el['help_text'] = (string)$this->type;
        return $el;
    }

    abstract function buildElement(): array;

    public function jsonSerialize(): mixed
    {
        return $this->buildElement();
    }
}

